var CALC_UI = new CalcUI();

function CalcUI() {
	
	// Private variables
	var maxOutputLines = 100;
	var minWidth = 280; // Try with Number.MAX_VALUE
	var minHeight = 135;
	var growboxInset;
	var scrollArea;
	var calculator = new Calc();
	var even = true;
	var nextID = 0;
	
	// For JavaScript event handlers
	var _self = this;
	
	// Help text
	var helpHTML = 
		'<b>Calculate</b> is a math expression calculator with persistent memory ' + 
		'for variables and functions. Any JavaScript expression can be evaluated.' +
		
		'<div class="helpHeader">Samples</div>' + 
		'<div class="input">2+2</div>' +
		'<div class="answer">4</div>' +
		'<div class="input">r=1/2</div>' +
		'<div class="answer">0.5</div>' +
		'<div class="input">2*pi*r</div>' +
		'<div class="answer">3.141592653589793</div>' +
		'<div class="input">cos(answer)</div>' +
		'<div class="answer">-1</div>' +
		'<div class="input">sqr = function(x) { return x*x }</div>' +
		'<div class="answer">Function defined</div>' +
		'<div class="input">sqr(3)</div>' +
		'<div class="answer">9</div>' +
		
		'<div class="helpHeader">Constants</div>' +
		"pi, e" +
		
		'<div class="helpHeader">Basic functions</div>' +
		"sqrt pow abs round floor ceil" +
		
		'<div class="helpHeader">Trig functions</div>' +
		"cos acos sin asin tan atan atan2" +
		
		'<div class="helpHeader">Log functions</div>' +
		"exp log" +
		
		'<div class="helpHeader">More info</div>' +
		'<ul>' + 
		'<li>Click the <b>x</b> button to clear the output window.</li>' +
		'<li>Click the <b>m</b> button to view the list of all constants, ' + 
			'variables, and functions.</li>' +
		'<li>Press the up and down arrow keys to browse previously entered expressions.</li>' +
		'<li>To delete a variable or function by name, enter "delete <i>name</i>".</li>' +
		
		'</ul>';
	
	// Private methods
	
	function scrollToEnd() {
		var output = document.getElementById('output');
		scrollArea.verticalScrollTo(output.scrollHeight);
		scrollArea.refresh();
	}
	
	function scrollTo(element) {
		scrollArea.reveal(element);
		scrollArea.refresh();
	}
	
	function outputHTML(html) {
		var output = document.getElementById('output');
		var id = "output" + nextID;
		
		output.innerHTML += 
			'<div id="' + id + '" class="' + (even?'even':'odd') + '">' +
			html + '</div>';
		even = !even;
		nextID++;
		
		// Trim output
		while (output.childNodes.length > maxOutputLines) {
			var node = output.childNodes[0];
			output.removeChild(node);
		}
		
		// Scroll to output
		scrollArea.refresh();
		scrollTo(document.getElementById(id));
	}
	

	// Privileged methods
	
	this.onLoad = function() {
		var scrollbar = new AppleVerticalScrollbar(document.getElementById("scrollBar"));
		scrollArea = new AppleScrollArea(document.getElementById("scrollArea"), scrollbar);
		scrollArea.scrollsHorizontally = false;
		scrollArea.singlepressScrollPixels = 25;
		
		document.addEventListener("mouseup", function(event) { 
			if (window.getSelection() == "") {
				_self.focusTextField() 
			}
		}, true);

		window.onfocus = function () { scrollArea.focus(); }
		window.onblur = function () { scrollArea.blur(); }
		
		
		if (window.widget) {
			widget.onhide = function() { calculator.saveUserVars(); }
			
			// Removed - keep vars in case of widget upgrade.
			//widget.onremove = function() { calculator.clearUserVars(); calculator.clearHistory(); }
			widget.onremove = function() { calculator.saveUserVars(); }
		}
		
		var edit = document.getElementById('edit');
		edit.value = "Enter an expression";
		edit.selectAll();
	
		scrollArea.refresh();
		_self.focusTextField();
	}	
	
	this.clearOutput = function() {
		var output = document.getElementById('output');
		output.innerHTML = "";
		even = true;
		scrollArea.refresh();
	}
	
	this.showHelp = function() {
		outputHTML('<div class="help">' + helpHTML + '</div>');
	}
	
	this.showMemory = function() {
		var userVars = calculator.getUserVars();
		var html = 
			'<div class="memory"><b>pi</b> = ' + Math.PI + "</div>" +
			'<div class="memory"><b>e</b> = ' + Math.E + "</div>" +
			'<div class="memory"><b>answer</b> = ' + calculator.getLastAnswer() + "</div>";
		
		for (var i in userVars) {
			var name = userVars[i];
			html += '<div class="memory"><b>' + name + "</b> = " + window[name] + "</div>";
		}
		
		outputHTML('<div class="help">' + html + '</div>');
	}
	
	this.clearMemory = function() {
		calculator.clearUserVars();
		calculator.clearHistory();
		
		var html = "Memory Cleared";
		outputHTML('<div class="help">' + html + '</div>');
	}

	this.focusTextField = function() {
		var edit = document.getElementById('edit');
		edit.focus();
	}
	
	this.resizeStart = function(event) {
	    document.addEventListener("mousemove", _self.resize, true);
	    document.addEventListener("mouseup", _self.resizeEnd, true);
	 
	    growboxInset = { x:(window.innerWidth - event.x), y:(window.innerHeight - event.y) };
	 
	    event.stopPropagation();
	    event.preventDefault();
	}
	 
	this.resize = function(event) {
		var width = Math.max(minWidth, event.x + growboxInset.x);
		var height = Math.max(minHeight, event.y + growboxInset.y);
	 
		window.resizeTo(width, height);
		scrollToEnd();
	 
		event.stopPropagation();
		event.preventDefault();
	}
	 
	this.resizeEnd = function(event) {
		document.removeEventListener("mousemove", _self.resize, true);
		document.removeEventListener("mouseup", _self.resizeEnd, true); 
		
		_self.focusTextField();
		
		event.stopPropagation();
		event.preventDefault();
	}
	
	this.keyRepeat = function(event) {
		var edit = document.getElementById('edit');
		var ch = String.fromCharCode(event.charCode);
	
		// History browsing
		if (event.keyIdentifier == "Up") {
			var prev = calculator.getHistoryPrev();
			if (prev != null) {
				edit.value = prev;
				edit.selectionStart = edit.value.length;
			}
		}
		else if (event.keyIdentifier == "Down") {
			var next = calculator.getHistoryNext();
			edit.value = next;
			edit.selectionStart = edit.value.length;
		}
		
		// First key handling
		else if (edit.value == "" && (ch == '+' || ch == '-' || ch == "*" || ch == "/" || ch == "%")) {
			edit.value = "answer" + ch;
			edit.selectionStart = edit.value.length;
		}
		
		// Enter expression
		else if (event.keyIdentifier == "Enter") {
			
			var addToHistory = true;
			
			if (edit.value == "") {
				// Use last expression
				var prev = calculator.getHistoryPrev();
				if (prev != null) {
					edit.value = prev;
					addToHistory = false;
				}
			}
			
			if (edit.value != "") {
				// Calculate
				var expression = edit.value;
				var result = calculator.calc(expression, addToHistory);
				edit.value = "";
	
				// Output results.
				var resultClass = calculator.wasError()?"error":"answer";
				outputHTML("<div class='input'>" + expression + "</div>" +
					"<div class='" + resultClass + "'>" + result + "</div>");
			}
		}
		
		// Normal key handling
		else {
			// Do nothing
			return;
		}
		
		event.stopPropagation();
		event.preventDefault();	
	}
}
